#include <stdio.h>
#include <sybfront.h>
#include <sybdb.h>
#include "sybdbex.h"
#include <wiscbench.h>

/*
 * wisc_query07.c
 *
 * Wisconsin Benchmark - single tuple selection to screen
 *                                
 * Implemented by David DeWitt and Michael Franklin
 * 
 * Copyright (c) 1991 David DeWitt and Michael Franklin
 *
 */

main (argc,argv)
    char	**argv;

{
    /* sybase specific variables */
    DBPROCESS       *dbproc;
    RETCODE        result_code; /* to hold the results of dbresults(). */
    STATUS         row_code;    /* to hold the results of dbnextrow(). */

    char	*username;
    char	*passwd;
    char	*optstring;
    extern 	char *optarg;
    extern 	int optind, opterr;
    int		c;
    int		errflg;

    int		i;
    int		attrValue;  
    int		tupCount; /* number of tuples in the relation */

    char	query[1000];
    int 	starttime, endtime, totaltime;
    int 	numtrials,totalrows,trialrows;

    username = passwd = NULL;
    errflg = 0;
    /* process the command line arguments */
    while ((c = getopt(argc,argv,"u:p:")) != EOF)
    {
	switch (c) {
	case 'u':	// get user name
	    username = optarg; 
	    break;
	case 'p':	// get user passwd
	    passwd = optarg;
	    break;
	case '?':
	    errflg++;
	}
	if (errflg) 
	{
	     usage();
	     exit (2);
	}
    }
    fprintf(stderr,"\nQuery 7 (1 tuple select, screen)...\n");

    /* next open the database */
    dbproc = open_db(DATABASE, username, passwd,FALSE);

    /* run query 7 - 1 tuple selection to screen */

    /* set base constants */
    tupCount = BASERELSIZE; /* from wiscbench.h */
    numtrials = 0;
    totaltime = 0;
    totalrows = 0;

    for (i=0;i<SELREPEATCNT/2;i++)
    {
	/* run SELREPEATCNT queries two at a time */

	/* first do one relation */
	attrValue = (int) random()%tupCount;
	sprintf(query,"select * from %s where unique2D = %d", 
		BASERELATION1, attrValue);
	if (VERBOSE) printf("next query is: %s\n",query);
	dbcmd(dbproc,query); /* form the query */

        /* Send the command buffer to SQL Server for execution. */
	starttime = time1000();
	dbsqlexec(dbproc);
        while ((result_code = dbresults(dbproc)) != NO_MORE_RESULTS) 
	{
	    if (result_code == SUCCEED) get_result_tuples(dbproc);
	}
    	endtime = time1000();

	/* update running statistics */
    	totaltime += (endtime-starttime);
    	trialrows = get_row_count(dbproc);
    	totalrows += trialrows;
    	print_trial(++numtrials,trialrows,(endtime-starttime)); 

	/* then do the second relation */
	attrValue = (int) random()%tupCount;
	sprintf(query,"select * from %s where unique2E = %d", 
		BASERELATION2, attrValue);
	dbcmd(dbproc,query); /* form the query */

    	/* Send the command buffer to SQL Server for execution. */
    	starttime = time1000();
    	dbsqlexec(dbproc);
    	while ((result_code = dbresults(dbproc)) != NO_MORE_RESULTS) 
	{
	    if (result_code == SUCCEED) get_result_tuples(dbproc);
	}
    	endtime = time1000();

	/* update running statistics */
    	totaltime += (endtime-starttime);
    	trialrows = get_row_count(dbproc);
    	totalrows += trialrows;
    	print_trial(++numtrials,trialrows,(endtime-starttime)); 
    }

    print_result("Query 7 (single tuple select to screen)",
	numtrials, totalrows, totaltime);
    closedb(dbproc);
    exit(STDEXIT);
}

/* =================================================================== */

usage ()
{
    fprintf (stderr,"\nUsage: wisc_query07 -u username -p passwd\n");
    fprintf (stderr,"\t-p: Show query plan\n");
    exit (1);
}
