#include <stdio.h>
#include <sybfront.h>
#include <sybdb.h>
#include "sybdbex.h"
#include <wiscbench.h>

/*
 * wisc_query20.c
 *
 * Wisconsin Benchmark - Scalar Minimum Aggregate
 *                       Query 20 (no index: use -n flag) or
 *                       Query 23 (clustered ix: use -i flag )
 *
 *                                
 *	NOTE: if -n used, then for valid results, this query must be run
 *		before running "wisc_index"
 *
 * Implemented by David DeWitt and Michael Franklin
 * 
 * Copyright (c) 1991 David DeWitt and Michael Franklin
 *
 */

main (argc,argv)
    char	**argv;

{
    /* sybase specific variables */
    DBPROCESS       *dbproc;
    RETCODE        result_code; /* to hold the results of dbresults(). */
    STATUS         row_code;    /* to hold the results of dbnextrow(). */

    char	*username;
    char	*passwd;
    char	*optstring;
    extern 	char *optarg;
    extern 	int optind, opterr;
    int		c;
    int		errflg;
    int		no_index;
    int		i;

    char	query[1000];
    int 	starttime, endtime, totaltime;
    int 	numtrials,totalrows,trialrows;

    username = passwd = NULL;
    errflg = 0;
    /* process the command line arguments */
    while ((c = getopt(argc,argv,"u:p:ni")) != EOF)
    {
	switch (c) {
	case 'u':	// get user name
	    username = optarg; 
	    break;
	case 'p':	// get user passwd
	    passwd = optarg;
	    break;
	case 'n':	// no indexing specified
	    no_index = TRUE;
	    break;
	case 'i':	// indexing specified
	    no_index = FALSE;
	    break;
	case '?':
	    errflg++;
	}
	if (errflg) 
	{
	     fprintf(stderr,"Must specifiy -n (no index) or -i (indexed) flag.\n");
	     usage();
	     exit (2);
	}
    }
    if (no_index) {
	    fprintf(stderr,"\nQuery 20 (NoIx, Scalar Min Aggregate)...\n");
    }
    else {
	    fprintf(stderr,"\nQuery 23 (ClusterIx, Scalar Min Aggregate)...\n");
	}

    /* next open the database */
    dbproc = open_db(DATABASE, username, passwd,FALSE);

    /* run query 20 or 23 - Scalar Min Aggregate*/

    /* set base constants */
    numtrials = 0;
    totaltime = 0;
    totalrows = 0;

    for (i=0;i<AGGREPEATCNT/2;i++)
    {
	/* run AGGREPEATCNT queries two at a time */

	/* first do one relation */
        create_agg_temp(dbproc);

	sprintf(query, 
	  "insert into TMP (uniqueval) select min(unique2D) from %s",
	  BASERELATION1);

	if (VERBOSE) printf("next query is: %s\n",query);
	dbcmd(dbproc,query); /* form the query */

        /* Send the command buffer to SQL Server for execution. */
	starttime = time1000();
	dbsqlexec(dbproc);
        while ((result_code = dbresults(dbproc)) != NO_MORE_RESULTS) continue;
    	endtime = time1000();

	/* update running statistics */
    	totaltime += (endtime-starttime);
    	trialrows = get_row_count(dbproc);
    	totalrows += trialrows;
    	drop_table(dbproc, "TMP"); /* delete the result relation */
    	print_trial(++numtrials,trialrows,(endtime-starttime)); 

	/* then do the second relation */
        create_agg_temp(dbproc);

	sprintf(query, 
	  "insert into TMP (uniqueval) select min(unique2E) from %s",
	  BASERELATION2);
	if (VERBOSE) printf("next query is: %s\n",query);
	dbcmd(dbproc,query); /* form the query */

    	/* Send the command buffer to SQL Server for execution. */
    	starttime = time1000();
    	dbsqlexec(dbproc);
    	while ((result_code = dbresults(dbproc)) != NO_MORE_RESULTS) continue;;
    	endtime = time1000();

	/* update running statistics */
    	totaltime += (endtime-starttime);
    	trialrows = get_row_count(dbproc);
    	totalrows += trialrows;
    	drop_table(dbproc, "TMP"); /* delete the result relation */
    	print_trial(++numtrials,trialrows,(endtime-starttime)); 
    }

    if (no_index) {
	 print_result("Query 20 (NoIx, Scalar Min Agg)",
		 numtrials, totalrows, totaltime);
    }
    else {
	 print_result("Query 23 (ClusterIx, Scalar Min Agg)",
		 numtrials, totalrows, totaltime);
    }
    closedb(dbproc);
    exit(STDEXIT);
}

/* =================================================================== */

usage ()
{
    fprintf (stderr,"\nUsage: wisc_query20 <-n or -i> -u username -p passwd\n");
    fprintf (stderr,"\t-n: There are no indices built on the database\n");
    fprintf (stderr,"\t-i: The database is indexed\n");
    fprintf (stderr,"\t-p: Show query plan\n");
    exit (1);
}
